import React from "react";
import { StyleSheet, Pressable, Text, View } from "react-native";
import tailwind from "tailwind-rn";
import { widthPercentageToDP as wp } from "react-native-responsive-screen";
import LinearGradient from "react-native-web-linear-gradient";

import { generateIconComponent } from "../../../shared/services";
import stylesSCSS from '../../../styles/style.module.scss';
import { IconNames } from "../../../shared/types-interfaces";

interface TuyaButtonProps {
  /**
   * Is this the principal call to action on the page?
   */
  primary?: boolean;
  /**
   * How large should the button be?
   */
  size?: "small" | "big" ;

  customWidth?: 'short' | 'large';
  /**
   * Button contents
   */
  label: string;
  /**
   * Button disable
   */
  disabled?: boolean;
  /**
   * Button vertical boolean
   */
  vertical?: boolean;
  /**
   * Vertical button shape
   */
  shape?: "square" | "circle";
  /**
   * Optional icon visibility
   */
  isIconVisible: boolean;
  /**
   * Optional icon width
   */
  iconWidth: string;
  /**
   * Optional icon height
   */
  iconHeight: string;
  /**
   * Optional icon name
   */
  iconName?: IconNames;
  /**
   * Optional click handler
   */
  onClick?: () => void;
}
export const TuyaButton: React.FC<TuyaButtonProps> = ({
  primary = false,
  size = "small",
  label,
  disabled = false,
  vertical = false,
  isIconVisible = false,
  iconWidth = "1em",
  iconHeight = "1em",
  iconName,
  shape = "square",
  customWidth = 'short',
  ...props
}: TuyaButtonProps) => {
  const styles = StyleSheet.create({
    verticalButton: {
      width: size === 'small' ? "100px": '106px',
      height: size === 'small' ? "100px": '106px',
    },
  });

  const getFontSize = (): string => size === 'small' ? 'text-sm' : 'text-base';
  const getDisableStyle = (): string => disabled ? "cursor-not-allowed opacity-50" : "opacity-100";
  const getMinWidth = (): string => customWidth === 'large' ? 'min-w-287' : size === 'small' ? 'min-w-118' : 'min-w-152';
  const getShapeStyle = ():string => shape === "circle" ? "rounded-full" : "rounded-lg";


  let dynamicIconComponent = generateIconComponent(iconName, {height: iconHeight, width: iconWidth});
  return !vertical ? (
    <View
      className={`flex flex-row items-center ${size === 'small'? 'h-11' : 'h-12'} ${getMinWidth()} ${getDisableStyle()}`}
    >
      {primary ? (
        <LinearGradient
          colors={[stylesSCSS.CTAtuyaColor1,stylesSCSS.CTAtuyaColor2]}
          style={[
            { marginHorizontal: wp(0) },
            tailwind(`h-full flex items-center justify-center rounded-2xl grow-0`),
          ]}
        >
          <Pressable
            className={`leading-none ${getMinWidth()} ${getDisableStyle()}`}
            onPress={() =>{
              props.onClick && props.onClick()
            }}
          >
            <View className={isIconVisible ? 'flex flex-row justify-center items-center': ''}>
              {isIconVisible && dynamicIconComponent}
              <Text className={`text-white font-bold text-center ${getFontSize()} ${isIconVisible ? 'ml-1': 'ml-0'}`}>
                {label}
              </Text>
            </View>
          </Pressable>
        </LinearGradient>
      ) : (
        <View
          style={[
            { marginHorizontal: wp(0) },
          ]}
          className={`leading-none h-full ${getMinWidth()}`}
        >
          <Pressable
            disabled={disabled}
            className={`bg-tuya-gray w-full h-full flex items-center justify-center self-center rounded-2xl grow-0 shadow-btn`}
            onPress={() =>{
              props.onClick && props.onClick()
            }}
          >
            <View className={isIconVisible ? 'flex flex-row justify-center items-center': ''}>
              {isIconVisible && dynamicIconComponent}
              <Text className={`${isIconVisible ? 'ml-1': 'ml-0'} text-red-600 font-bold text-center ${getFontSize()}`}>
                {label}
              </Text>
            </View>
          </Pressable>
        </View>
      )}
    </View>
  ) : (
    <View
      style={[
        { marginHorizontal: wp(0) },
        styles.verticalButton,
      ]}
      className={`opacity-100 bg-tuya-gray shadow-btn ${getShapeStyle()} ${getDisableStyle()}`}
    >
      <Pressable
        style={[styles.verticalButton]}
        disabled={disabled}
        className={`bg-tuya-gray shadow-btn flex flex-col text-center justify-center align-middle items-center ${getShapeStyle()} ${getDisableStyle()}
        `}
        onPress={() =>{
          props.onClick && props.onClick()
        }}
      >
        {isIconVisible && dynamicIconComponent}

        <Text
          className={`px-3 py-1 text-center whitespace-normal text-black text-ellipsis overflow-hidden leading-5 ${getFontSize()} `}
        >
          {label}
        </Text>
      </Pressable>
    </View>
  );
};
